<script setup lang="ts">
import type { Recordable } from '@vben/types';

import { onMounted, ref } from 'vue';

import { Page, useVbenDrawer, type VbenFormProps } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { addFullName, getPopupContainer, getVxePopupContainer } from '@vben/utils';

import { Dropdown, Menu, MenuItem, Modal, Popconfirm, Space } from 'ant-design-vue';

import { useVbenVxeGrid, vxeCheckboxChecked, type VxeGridProps, vxeSortEvent } from '#/adapter/vxe-table';
import { delMaintPlan, listMaintPlan, maintPlanExport } from '#/api/eims/maint-plan';
import { generateCode } from '#/api/eims/utils';
import { getDeptTree, userList } from '#/api/system/user';
import { MAINT_ORDER_STATUS } from '#/constants/dict';
import { commonDownloadExcel } from '#/utils/file/download';
import BasisSubTable from '#/views/eims/components/basis-sub-table.vue';
import maintOrderDrawer from '#/views/eims/maint-order/maint-order-drawer.vue';

import { columns, querySchema } from './data';
import maintPlanDrawer from './maint-plan-drawer.vue';


defineExpose({
  tableSelect
});

const formOptions: VbenFormProps = {
  commonConfig: {
    labelWidth: 80,
    componentProps: {
      allowClear: true
    }
  },
  collapsed: true,
  schema: querySchema(),
  wrapperClass: 'grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4'
};

const gridOptions: VxeGridProps = {
  checkboxConfig: {
    // 高亮
    highlight: true,
    // 翻页时保留选中状态
    reserve: true
    // 点击行选中
    // trigger: 'row'
  },
  columns,
  height: 'auto',
  keepSource: true,
  pagerConfig: {},
  proxyConfig: {
    ajax: {
      query: async ({ page }, formValues = {}) => {
        return await listMaintPlan({
          pageNum: page.currentPage,
          pageSize: page.pageSize,
          ...formValues
        });
      }
    }
  },
  rowConfig: {
    isHover: true,
    keyField: 'id'
  },
  sortConfig: {
    // 远程排序
    remote: true,
    // 支持多字段排序 默认关闭
    multiple: true
  },
  id: 'maint-plan-index'
};
const equId = ref<string>();
const [BasicTable, tableApi] = useVbenVxeGrid({
  formOptions,
  gridOptions,
  gridEvents: {
    sortChange: (sortParams) => vxeSortEvent(tableApi, sortParams),
    cellClick: (e: any) => {
      const { row } = e;
      equId.value = row.equId;
    }
  }
});

const [MaintPlanDrawer, maintPlanDrawerApi] = useVbenDrawer({
  connectedComponent: maintPlanDrawer
});

const [MaintOrderDrawer, maintOrderDrawerApi] = useVbenDrawer({
  connectedComponent: maintOrderDrawer,
});

function handleAdd() {
  maintPlanDrawerApi.setData({});
  maintPlanDrawerApi.open();
}

async function handleEdit(record: Recordable<any>) {
  maintPlanDrawerApi.setData({ id: record.id });
  maintPlanDrawerApi.open();
}

async function handleDelete(row: Recordable<any>) {
  await delMaintPlan(row.id);
  await tableApi.query();
}

function handleMultiDelete() {
  const rows = tableApi.grid.getCheckboxRecords();
  const ids = rows.map((row: any) => row.id);
  Modal.confirm({
    title: '提示',
    okType: 'danger',
    content: `确认删除选中的${ids.length}条记录吗？`,
    onOk: async () => {
      await delMaintPlan(ids);
      await tableApi.query();
    }
  });
}

function handleDownloadExcel() {
  commonDownloadExcel(maintPlanExport, '保养计划记录', tableApi.formApi.form.values, {
    fieldMappingTime: formOptions.fieldMappingTime
  });
}

/**
 * 生成保养工单
 * @param record
 */
async function handleAddOrder(record: Recordable<any>) {
  /**
   * 生成保养工单code
   */
  const code = await generateCode('BYYJ');
  if (!code) {
    Modal.error({
      content: '生成保养工单失败，请重试！',
      title: '提示'
    });
    return false;
  }
  const planOrder = {
    equId: record.equId,
    planId: record.id,
    equName: record.equName,
    maintUser: record.maintUser,
    maintUserName: record.maintUserName,
    maintDept: record.maintDept,
    maintDeptName: record.maintDeptName,
    maintType: record.maintType,
    planTime: record.maintNextTime,
    status: MAINT_ORDER_STATUS.DAIBAOYANG,
    maintCode: code
  };

  maintOrderDrawerApi.setData({ planOrder });
  maintOrderDrawerApi.open();
  // await addMaintOrder(order.value);
}
/**
 * 初始化部门选择
 */
async function setupDeptSelect() {
  // updateSchema
  const deptTree = await getDeptTree();
  // 选中后显示在输入框的值 即父节点 / 子节点
  addFullName(deptTree, 'label', ' / ');
  tableApi.formApi.updateSchema([
    {
      componentProps: (formModel) => ({
        class: 'w-full',
        fieldNames: {
          key: 'id',
          value: 'id',
          children: 'children'
        },
        getPopupContainer,
        async onSelect(deptId: number | string) {
          /** 根据部门ID加载用户 */
          await setupUserOptions(deptId);
          /** 变化后需要重新选择用户 */
          formModel.maintUser = undefined;
        },
        placeholder: '请选择',
        showSearch: true,
        treeData: deptTree,
        treeDefaultExpandAll: true,
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'label',
        // 选中后显示在输入框的值
        treeNodeLabelProp: 'fullName'
      }),
      fieldName: 'maintDept'
    }
  ]);
}

/**
 * 用户的加载
 */
async function setupUserOptions(deptId: any) {
  const params = { deptId };
  const userPageResult = await userList({
    pageNum: 1,
    pageSize: 500,
    ...params
  });
  const options = userPageResult.rows.map((item) => ({
    label: item.nickName || item.userName,
    value: item.userId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该部门下暂无用户';
  tableApi.formApi.updateSchema([
    {
      componentProps: { options, placeholder, filterOption },
      fieldName: 'maintUser'
    }
  ]);
}

onMounted(async () => {
  await setupDeptSelect();
});

// 选中数据
function tableSelect() {
  return tableApi.grid.getCheckboxRecords();
}
</script>

<template>
  <Page :auto-content-height="true">
    <div class="flex h-full gap-[8px] flex-col">
      <BasicTable  table-title="保养计划列表">
        <template #toolbar-tools>
          <Space>
            <a-button v-access:code="['eims:maintPlan:export']" @click="handleDownloadExcel">
              {{ $t('pages.common.export') }}
            </a-button>
            <a-button
              :disabled="!vxeCheckboxChecked(tableApi)"
              danger
              type="primary"
              v-access:code="['eims:maintPlan:remove']"
              @click="handleMultiDelete"
            >
              {{ $t('pages.common.delete') }}
            </a-button>
            <a-button type="primary" v-access:code="['eims:maintPlan:add']" @click="handleAdd">
              {{ $t('pages.common.add') }}
            </a-button>
          </Space>
        </template>

        <template #equName="{ row }">
          <Space>
            <span>{{ row.equName }}</span>
          </Space>
        </template>

        <template #action="{ row }">
          <Space>
            <ghost-button class="btn-success" v-access:code="['eims:maintPlan:edit']" @click.stop="handleAddOrder(row)"> 生成工单 </ghost-button>
            <ghost-button v-access:code="['eims:maintPlan:edit']" @click.stop="handleEdit(row)">
              {{ $t('pages.common.edit') }}
            </ghost-button>
          </Space>
          <Dropdown :get-popup-container="getVxePopupContainer" placement="bottomRight">
            <template #overlay>
              <Menu>
                <Popconfirm :get-popup-container="getVxePopupContainer" placement="left" title="确认删除？" @confirm="handleDelete(row)">
                  <MenuItem key="1" @click.stop="">
                    {{ $t('pages.common.delete') }}
                  </MenuItem>
                </Popconfirm>
              </Menu>
            </template>
            <a-button size="small" type="link" v-access:code="['eims:maintPlan:remove']">
              {{ $t('pages.common.more') }}
            </a-button>
          </Dropdown>
        </template>
      </BasicTable>
    </div>
    <MaintPlanDrawer @reload="tableApi.query()" />
    <MaintOrderDrawer @reload="tableApi.query()" />
  </Page>
</template>
